﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Threading;
using System.Diagnostics;
using System.Windows.Media;

using Styx;
using Styx.Helpers;
using Styx.Plugins;
using Styx.Common;
using Styx.CommonBot;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace ReadySteadyPlugins
{
    public class ReadySteadyPlugins : HBPlugin
    {
        public override string Name { get { return "Ready Steady Plugins"; } }
        public override string Author { get { return "Rozman"; } }
        public override Version Version { get { return new Version(1, 3); } }
        public override bool WantButton { get { return true; } }
        private static LocalPlayer Me { get { return StyxWoW.Me; } }

        private static readonly Stopwatch timerLFGAccept = new Stopwatch();
        private static readonly Stopwatch timerReadyCheckAccept = new Stopwatch();
        private static readonly Stopwatch timerResurrectAccept = new Stopwatch();
        private static readonly Stopwatch timerRoleCheckAccept = new Stopwatch();
        private static readonly Stopwatch timerBGArenaAccept = new Stopwatch();

        private bool AcceptLFGTrigger = false;
        private bool AcceptReadyCheckTrigger = false;
        private bool AcceptResurrectTrigger = false;
        private bool AcceptRoleCheckTrigger = false;
        private bool AcceptBGArenaTrigger = false;

        private int AcceptingLFGTime = 0;
        private int AcceptingReadyCheckTime = 0;
        private int AcceptingResurrectTime = 0;
        private int AcceptingRoleCheckTime = 0;
        private int AcceptingBGArenaTime = 0;

        public override void OnEnable()
        {
            Lua.Events.AttachEvent("LFG_PROPOSAL_SHOW", AcceptLFG);
            Lua.Events.AttachEvent("READY_CHECK", Ready);
            Lua.Events.AttachEvent("RESURRECT_REQUEST", AcceptRes);
            Lua.Events.AttachEvent("LFG_ROLE_CHECK_SHOW", RoleCheck);
            Lua.Events.AttachEvent("UPDATE_BATTLEFIELD_STATUS", BGArena);

            Logging.Write("ReadySteadyPlugins - Loaded Version " + Version);
        }

        public override void OnDisable()
        {
            Lua.Events.DetachEvent("LFG_PROPOSAL_SHOW", AcceptLFG);
            Lua.Events.DetachEvent("READY_CHECK", Ready);
            Lua.Events.DetachEvent("RESURRECT_REQUEST", AcceptRes);
            Lua.Events.DetachEvent("LFG_ROLE_CHECK_SHOW", RoleCheck);
            Lua.Events.DetachEvent("UPDATE_BATTLEFIELD_STATUS", BGArena);

            Logging.Write("ReadySteadyPlugins - Disabled");

            timerLFGAccept.Stop();
            timerLFGAccept.Reset();
            AcceptLFGTrigger = false;
            AcceptingLFGTime = 0;

            timerReadyCheckAccept.Stop();
            timerReadyCheckAccept.Reset();
            AcceptReadyCheckTrigger = false;
            AcceptingReadyCheckTime = 0;

            timerResurrectAccept.Stop();
            timerResurrectAccept.Reset();
            AcceptResurrectTrigger = false;
            AcceptingResurrectTime = 0;

            timerRoleCheckAccept.Stop();
            timerRoleCheckAccept.Reset();
            AcceptRoleCheckTrigger = false;
            AcceptingRoleCheckTime = 0;
            
            timerBGArenaAccept.Stop();
            timerBGArenaAccept.Reset();
            AcceptBGArenaTrigger = false;
            AcceptingBGArenaTime = 0;
        }

        public override void Pulse()
        {
			if (!StyxWoW.IsInGame) return;
            if (!AcceptLFGTrigger && !AcceptReadyCheckTrigger && !AcceptResurrectTrigger && !AcceptRoleCheckTrigger && !AcceptBGArenaTrigger) return;

            //LFG, LFR
            if (AcceptLFGTrigger) {
                if (!timerLFGAccept.IsRunning) timerLFGAccept.Start();

                if (timerLFGAccept.ElapsedMilliseconds < AcceptingLFGTime)
                {
                    return;
                }

                Lua.DoString("AcceptProposal()");

                timerLFGAccept.Stop();
                timerLFGAccept.Reset();
                AcceptLFGTrigger = false;
                AcceptingLFGTime = 0;
            }

            //ReadyCheck
            if (AcceptReadyCheckTrigger) {
                if (!timerReadyCheckAccept.IsRunning) timerReadyCheckAccept.Start();

                if (timerReadyCheckAccept.ElapsedMilliseconds < AcceptingReadyCheckTime)
                {
                    return;
                }

                if (Me.IsDead || Me.IsGhost)
                {
                    Logging.Write("Clicked NOT READY, because we are dead.");
                    Lua.DoString("RunMacroText(\"/click ReadyCheckFrameNoButton\");");
                } else {
                    Lua.DoString("RunMacroText(\"/click ReadyCheckFrameYesButton\");");
                    Lua.DoString("/script ConfirmReadyCheck(1);");
                }

                timerReadyCheckAccept.Stop();
                timerReadyCheckAccept.Reset();
                AcceptReadyCheckTrigger = false;
                AcceptingReadyCheckTime = 0;
            }

            //Resurrection
            if (AcceptResurrectTrigger) {
                if (!timerResurrectAccept.IsRunning) timerResurrectAccept.Start();

                if (timerResurrectAccept.ElapsedMilliseconds < AcceptingResurrectTime)
                {
                    return;
                }

                Lua.DoString("RunMacroText(\"/click StaticPopup1Button1\");");
                Lua.DoString("/script AcceptResurrect();");

                timerResurrectAccept.Stop();
                timerResurrectAccept.Reset();
                AcceptResurrectTrigger = false;
                AcceptingResurrectTime = 0;
            }

            //RoleCheck
            if (AcceptRoleCheckTrigger) {
                if (!timerRoleCheckAccept.IsRunning) timerRoleCheckAccept.Start();

                if (timerRoleCheckAccept.ElapsedMilliseconds < AcceptingRoleCheckTime)
                {
                    return;
                }

                if (Styx.StyxWoW.Me.SpecType == SpecType.Healer)
                {
                    Lua.DoString("/run UnitSetRole(\"player\",\"HEALER\")");
                    Lua.DoString("/script UnitSetRole(\"player\",\"HEALER\")");
                    Logging.Write("[ReadySteadyPlugins]: checked role as HEALER");
                }
                else if (Styx.StyxWoW.Me.SpecType == SpecType.RangedDps || Styx.StyxWoW.Me.SpecType == SpecType.MeleeDps)
                {
                    Lua.DoString("/run UnitSetRole(\"player\",\"DAMAGER\")");
                    Lua.DoString("/script UnitSetRole(\"player\",\"DAMAGER\")");
                    Logging.Write("[ReadySteadyPlugins]: checked role as DAMAGER");
                }
                else if (Styx.StyxWoW.Me.SpecType == SpecType.Tank)
                {
                    Lua.DoString("/run UnitSetRole(\"player\",\"TANK\")");
                    Lua.DoString("/script UnitSetRole(\"player\",\"TANK\")");
                    Logging.Write("[ReadySteadyPlugins]: checked role as TANK");

                }

                timerRoleCheckAccept.Stop();
                timerRoleCheckAccept.Reset();
                AcceptRoleCheckTrigger = false;
                AcceptingRoleCheckTime = 0;
            }

            //BG or ARena
            if (AcceptBGArenaTrigger) {

                // TODO: use MAX_BATTLEFIELD_QUEUES
                for (int i = 1; i <= 3; i++) {
                    List<string> BGStatus = Lua.GetReturnValues("return GetBattlefieldStatus(" + i + ")");

                    if (BGStatus[0] == "confirm") {
                        Logging.Write("[ReadySteadyPlugins]: Yes, it's confirm window. Accepting between 4-25 seconds");

                        if (!timerBGArenaAccept.IsRunning) timerBGArenaAccept.Start();

                        if (timerBGArenaAccept.ElapsedMilliseconds < AcceptingBGArenaTime)
                        {
                            return;
                        }

                        Lua.DoString("AcceptBattlefieldPort("+i+",1)");
                        Lua.DoString("StaticPopup_Hide(\"CONFIRM_BATTLEFIELD_ENTRY\")");
                        break;
                    }
                }

                timerBGArenaAccept.Stop();
                timerBGArenaAccept.Reset();
                AcceptBGArenaTrigger = false;
                AcceptingBGArenaTime = 0;
            }
        }

		public void AcceptLFG(object sender, LuaEventArgs args)
		{
            Random r = new Random();

			AcceptLFGTrigger = true;

			AcceptingLFGTime = r.Next(4000, 20000);

			Logging.Write("[ReadySteadyPlugins]: LFG menu detected, accepting between 4-20 seconds");
		}

        public void Ready(object sender, LuaEventArgs args)
        {
            Random r = new Random();

            AcceptReadyCheckTrigger = true;

            AcceptingReadyCheckTime = r.Next(2000, 6000);

            Logging.Write("[ReadySteadyPlugins]: Ready Check menu detected, clicking Ready between 2-6 seconds");
        }

        public void AcceptRes(object sender, LuaEventArgs args)
        {
            Random r = new Random();

            AcceptResurrectTrigger = true;

            AcceptingResurrectTime = r.Next(3000, 6000);

            Logging.Write("[ReadySteadyPlugins]: Resurrect menu detected, accepting between 3-6 seconds");
        }

        public void RoleCheck(object sender, LuaEventArgs args)
        {
            Random r = new Random();

			AcceptRoleCheckTrigger = true;

			AcceptingRoleCheckTime = r.Next(4000, 8000);

            Logging.Write("[ReadySteadyPlugins]: Role Check menu detected, clicking Ready between 4-8 seconds");
        }

        public void BGArena(object sender, LuaEventArgs args)
        {
            Random r = new Random();

			AcceptBGArenaTrigger = true;

			AcceptingBGArenaTime = r.Next(4000, 25000);

            Logging.Write("[ReadySteadyPlugins]: BG/Arena status changed. I will go and look, it's CONFIRM window or not...");
        }
    }
}